@@ -23,11 +23,12 @@
 
 #include "samplv1widget_lv2.h"
 
+#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
-#include "lv2/lv2plug.in/ns/ext/event/event-helpers.h"
+#include "lv2/lv2plug.in/ns/ext/atom/util.h"
+
 #include "lv2/lv2plug.in/ns/ext/instance-access/instance-access.h"
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 
 #include <unistd.h>
 
@@ -44,14 +45,14 @@ samplv1_lv2::samplv1_lv2 (
 	: samplv1(2, uint32_t(sample_rate))
 {
 	m_urid_map = NULL;
-	m_event_id = 0;
-	m_event_buffer = NULL;
+	m_midi_event_type = 0;
+	m_atom_sequence = NULL;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		if (::strcmp(host_features[i]->URI, LV2_URID_MAP_URI) == 0) {
 			m_urid_map = (LV2_URID_Map *) host_features[i]->data;
 			if (m_urid_map) {
-				m_event_id = m_urid_map->map(
+				m_midi_event_type = m_urid_map->map(
 					m_urid_map->handle, LV2_MIDI__MidiEvent);
 				break;
 			}
@@ -80,7 +81,7 @@ void samplv1_lv2::connect_port ( uint32_t port, void *data )
 {
 	switch(PortIndex(port)) {
 	case MidiIn:
-		m_event_buffer = (LV2_Event_Buffer *) data;
+		m_atom_sequence = (LV2_Atom_Sequence *) data;
 		break;
 	case AudioInL:
 		m_ins[0] = (float *) data;
@@ -112,14 +113,11 @@ void samplv1_lv2::run ( uint32_t nframes )
 
 	uint32_t ndelta = 0;
 
-	if (m_event_buffer) {
-		LV2_Event_Iterator iter;
-		lv2_event_begin(&iter, m_event_buffer);
-		while (lv2_event_is_valid(&iter)) {
-			uint8_t   *data;
-			LV2_Event *event = lv2_event_get(&iter, &data);
-			if (event && event->type == m_event_id) {
-				uint32_t nread = event->frames - ndelta;
+	if (m_atom_sequence) {
+		LV2_ATOM_SEQUENCE_FOREACH(m_atom_sequence, event) {
+			if (event && event->body.type == m_midi_event_type) {
+				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
+				uint32_t nread = event->time.frames - ndelta;
 				if (nread > 0) {
 					process(ins, outs, nread);
 					for (uint16_t k = 0; k < nchannels; ++k) {
@@ -127,12 +125,11 @@ void samplv1_lv2::run ( uint32_t nframes )
 						outs[k] += nread;
 					}
 				}
-				ndelta = event->frames;
-				process_midi(data, event->size);
+				ndelta = event->time.frames;
+				process_midi(data, event->body.size);
 			}
-			lv2_event_increment(&iter);
 		}
-	//	m_event_buffer = NULL;
+	//	m_atom_sequence = NULL;
 	}
 
 	process(ins, outs, nframes - ndelta);
@@ -226,6 +223,25 @@ static LV2_State_Status samplv1_lv2_state_save ( LV2_Handle instance,
 	if (map_path)
 		::free((void *) value);
 
+	// Save extra loop points...
+	uint32_t loop_start = pPlugin->loopStart();
+	uint32_t loop_end   = pPlugin->loopEnd();
+
+	if (loop_start < loop_end) {
+		type = pPlugin->urid_map(LV2_ATOM__Int);
+		if (type) {
+			size = sizeof(uint32_t);
+			value = (const char *) &loop_start;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+			if (key)
+				(*store)(handle, key, value, size, type, flags);
+			value = (const char *) &loop_end;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+			if (key)
+				(*store)(handle, key, value, size, type, flags);
+		}
+	}
+
 	return result;
 }
 
@@ -281,11 +297,39 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 		return LV2_STATE_ERR_UNKNOWN;
 
 	pPlugin->setSampleFile((const char *) value);
-	pPlugin->update_notify();
 
 	if (map_path)
 		::free((void *) value);
 
+	// Restore extra loop points.
+	uint32_t loop_start = 0;
+	uint32_t loop_end   = 0;
+
+	uint32_t int_type = pPlugin->urid_map(LV2_ATOM__Int);
+	if (int_type) {
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+			if (value && size == sizeof(uint32_t) && type == int_type)
+				loop_start = *(uint32_t *) value;
+		}
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+			if (value && size == sizeof(uint32_t) && type == int_type)
+				loop_end = *(uint32_t *) value;
+		}
+	}
+
+	if (loop_start < loop_end)
+		pPlugin->setLoopRange(loop_start, loop_end);
+
+	pPlugin->update_notify();
+
 	return LV2_STATE_SUCCESS;
 }
 
