@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -100,7 +100,7 @@ struct {
 	{ "DEL1_WET",       0.0f },
 	{ "DEL1_DELAY",     0.5f },
 	{ "DEL1_FEEDB",     0.5f },
-	{ "DEL1_BPM",       1.8f },
+	{ "DEL1_BPM",     180.0f },
 	{ "DYN1_COMPRESS",  0.0f },
 	{ "DYN1_LIMIT",     1.0f }
 };
@@ -238,8 +238,9 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Out1PanningKnob->setMaximum(+1.0f);
 
 	// Effects (delay BPM)
-	m_ui.Del1BpmKnob->setMinimum(0.0f);
-	m_ui.Del1BpmKnob->setMaximum(3.6f);
+	m_ui.Del1BpmKnob->setScale(1.0f);
+	m_ui.Del1BpmKnob->setMinimum(3.6f);
+	m_ui.Del1BpmKnob->setMaximum(360.0f);
 
 
 	// GEN1
@@ -568,16 +569,28 @@ void samplv1widget::paramChanged ( float fValue )
 // Update local tied widgets.
 void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 {
+	samplv1 *pSampl = instance();
+	if (pSampl == NULL)
+		return;
+
+	++m_iUpdate;
+
 	switch (index) {
 	case samplv1::GEN1_LOOP: {
 		const bool bLoop = bool(fValue > 0.0f);
+		pSampl->setLoop(bLoop);
 		m_ui.Gen1Sample->setLoop(bLoop);
+		m_ui.Gen1Sample->setLoopStart(pSampl->loopStart());
+		m_ui.Gen1Sample->setLoopEnd(pSampl->loopEnd());
 		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
+		updateSampleLoop(pSampl->sample());
 		// Fall thru...
 	}
 	default:
 		break;
 	}
+
+	--m_iUpdate;
 }
 
 
@@ -756,6 +769,10 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 							if (!sName.isEmpty() && s_hash.contains(sName))
 								index = s_hash.value(sName);
 							float fValue = eParam.text().toFloat();
+						//--legacy support < 0.3.0.4 -- begin
+							if (index == samplv1::DEL1_BPM && fValue < 3.6f)
+								fValue *= 100.0f;
+						//--legacy support < 0.3.0.4 -- end.
 							setParamValue(index, fValue);
 							updateParam(index, fValue);
 							m_params_ab[index] = fValue;
@@ -859,7 +876,13 @@ void samplv1widget::loadSamples ( const QDomElement& eSamples )
 				sFilename = eSample.text();
 			// Done it.
 			loadSampleFile(sFilename);
-			setSampleLoop(iLoopStart, iLoopEnd);
+			// Set actual sample loop points...
+			pSampl->setLoopRange(iLoopStart, iLoopEnd);
+			++m_iUpdate;
+			m_ui.Gen1Sample->setLoopStart(iLoopStart);
+			m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
+			updateSampleLoop(pSampl->sample());
+			--m_iUpdate;
 		}
 	}
 
@@ -973,20 +996,17 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 	++m_iUpdate;
 	if (pSample) {
 		const bool bLoop = pSample->isLoop();
-		const uint32_t nframes = pSample->length();
 		m_ui.Gen1Sample->setLoop(bLoop);
-		m_ui.Gen1Sample->setLoopStart(0);
-		m_ui.Gen1Sample->setLoopEnd(nframes);
-		m_ui.Gen1LoopStartSpinBox->setMaximum(nframes);
-		m_ui.Gen1LoopStartSpinBox->setValue(0);
-		m_ui.Gen1LoopEndSpinBox->setMaximum(nframes);
-		m_ui.Gen1LoopEndSpinBox->setValue(nframes);
+		const uint32_t iLoopStart = pSample->loopStart();
+		const uint32_t iLoopEnd = pSample->loopEnd();
+		m_ui.Gen1Sample->setLoopStart(iLoopStart);
+		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
+		updateSampleLoop(pSample);
 	} else {
 		m_ui.Gen1Sample->setLoop(false);
-		m_ui.Gen1LoopStartSpinBox->setMaximum(0);
-		m_ui.Gen1LoopStartSpinBox->setValue(0);
-		m_ui.Gen1LoopEndSpinBox->setMaximum(0);
-		m_ui.Gen1LoopEndSpinBox->setValue(0);
+		m_ui.Gen1Sample->setLoopStart(0);
+		m_ui.Gen1Sample->setLoopEnd(0);
+		updateSampleLoop(NULL);
 	}
 	--m_iUpdate;
 
@@ -995,29 +1015,6 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 }
 
 
-// Sample loop points.
-void samplv1widget::setSampleLoop ( uint32_t iLoopStart, uint32_t iLoopEnd )
-{
-	if (iLoopStart < iLoopEnd) {
-		++m_iUpdate;
-		samplv1 *pSampl = instance();
-		if (pSampl) {
-			pSampl->setLoopRange(iLoopStart, iLoopEnd);
-			m_ui.Gen1Sample->setLoopStart(iLoopStart);
-			m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-			const uint32_t nframes = pSampl->sample()->length();
-			m_ui.Gen1LoopStartSpinBox->setMaximum(
-				iLoopEnd > 0 ? iLoopEnd - 1 : 0);
-			m_ui.Gen1LoopEndSpinBox->setMinimum(
-				iLoopStart < nframes ? iLoopStart + 1 : nframes);
-			m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
-			m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
-		}
-		--m_iUpdate;
-	}
-}
-
-
 // MIDI note/octave name helper (static).
 QString samplv1widget::noteName ( int note )
 {
@@ -1046,14 +1043,7 @@ void samplv1widget::loopRangeChanged (void)
 		const uint32_t iLoopStart = m_ui.Gen1Sample->loopStart();
 		const uint32_t iLoopEnd = m_ui.Gen1Sample->loopEnd();
 		pSampl->setLoopRange(iLoopStart, iLoopEnd);
-		const uint32_t nframes = pSampl->sample()->length();
-		m_ui.Gen1LoopStartSpinBox->setMaximum(
-			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
-		m_ui.Gen1LoopEndSpinBox->setMinimum(
-			iLoopStart < nframes ? iLoopStart + 1 : nframes);
-		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
-		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
-		updateLoopRange(iLoopStart, iLoopEnd);
+		updateSampleLoop(pSampl->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -1072,10 +1062,7 @@ void samplv1widget::loopStartChanged (void)
 		const uint32_t iLoopEnd = pSampl->loopEnd();
 		pSampl->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopStart(iLoopStart);
-		const uint32_t nframes = pSampl->sample()->length();
-		m_ui.Gen1LoopEndSpinBox->setMinimum(
-			iLoopStart < nframes ? iLoopStart + 1 : nframes);
-		updateLoopRange(iLoopStart, iLoopEnd);
+		updateSampleLoop(pSampl->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -1094,20 +1081,40 @@ void samplv1widget::loopEndChanged (void)
 		const uint32_t iLoopEnd = m_ui.Gen1LoopEndSpinBox->value();
 		pSampl->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-		m_ui.Gen1LoopStartSpinBox->setMaximum(
-			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
-		updateLoopRange(iLoopStart, iLoopEnd);
+		updateSampleLoop(pSampl->sample(), true);
 	}
 	--m_iUpdate;
 }
 
 
-void samplv1widget::updateLoopRange ( uint32_t iLoopStart, uint32_t iLoopEnd )
+void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 {
-	m_ui.Preset->dirtyPreset();
-	m_ui.StatusBar->showMessage(tr("Loop start: %1, end: %2")
-		.arg(iLoopStart).arg(iLoopEnd), 5000);
-	m_ui.StatusBar->setModified(true);
+	if (pSample) {
+		const uint32_t iLoopStart = pSample->loopStart();
+		const uint32_t iLoopEnd = pSample->loopEnd();
+		const uint32_t nframes = pSample->length();
+		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(
+			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
+		m_ui.Gen1LoopEndSpinBox->setMinimum(
+			iLoopStart < nframes ? iLoopStart + 1 : nframes);
+		m_ui.Gen1LoopEndSpinBox->setMaximum(nframes);
+		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
+		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
+		if (bDirty) {
+			m_ui.Preset->dirtyPreset();
+			m_ui.StatusBar->showMessage(tr("Loop start: %1, end: %2")
+				.arg(iLoopStart).arg(iLoopEnd), 5000);
+			m_ui.StatusBar->setModified(true);
+		}
+	} else {
+		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(0);
+		m_ui.Gen1LoopStartSpinBox->setValue(0);
+		m_ui.Gen1LoopEndSpinBox->setMinimum(0);
+		m_ui.Gen1LoopEndSpinBox->setMaximum(0);
+		m_ui.Gen1LoopEndSpinBox->setValue(0);
+	}
 }
 
 
